/********************************************************************/
/*                                                                  */
/* Licensed Materials - Property of IBM                             */
/* IBM(R) XL C/C++ Advanced Edition V7.0 for Linux(R)               */
/* 5724-K77                                                         */
/* (C) Copyright IBM Corp. 2003, 2004. All Rights Reserved.         */
/* US Government Users Restricted Rights - Use, duplication or      */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.*/
/*                                                                  */
/********************************************************************/

#ifndef __IBM_VACPP_math_h
#define __IBM_VACPP_math_h 1

#if defined __MATH__ /* && ! defined __NO_MATH_INLINES */

# ifdef __cplusplus

#   include <features.h>
#   include <sys/types.h>   

extern "C" {
    extern double acos(double) __THROW __attribute__((__IBMBuiltin__));
    extern double asin(double) __THROW __attribute__((__IBMBuiltin__));
    extern double atan(double) __THROW __attribute__((__IBMBuiltin__));
    extern double atan2(double, double) __THROW __attribute__((__IBMBuiltin__));
    extern double ceil(double) __THROW __attribute__((__IBMBuiltin__));
    extern double cos(double) __THROW __attribute__((__IBMBuiltin__));
    extern double exp(double) __THROW __attribute__((__IBMBuiltin__));
    extern double fabs(double) __THROW __attribute__((__IBMBuiltin__));
    extern double floor(double) __THROW __attribute__((__IBMBuiltin__));
    extern double log(double) __THROW __attribute__((__IBMBuiltin__));
    extern double log10(double) __THROW __attribute__((__IBMBuiltin__));
    extern double sin(double) __THROW __attribute__((__IBMBuiltin__));
    extern double sqrt(double) __THROW __attribute__((__IBMBuiltin__));
    extern double tan(double) __THROW __attribute__((__IBMBuiltin__));
    
#   ifdef _ALL_SOURCE
    extern double copysign(double, double) __THROW __attribute__((__IBMBuiltin__));
#   endif   
};

# endif /* __cplusplus */

#endif /* defined __MATH && ! defined __NO_MATH_INLINES */

#include_next <math.h>

#if defined __MATH__ /* && !defined __NO_MATH_INLINES */
     
# ifndef __cplusplus /* ! __cplusplus */

#   define acos(__x)         __acos(__x)
#   define asin(__x)         __asin(__x)
#   define atan(__x)         __atan(__x)
#   define atan2(__x,__y)    __atan2(__x,__y)
#   define ceil(__x)         __ceil(__x)
#   define ceilf(__x)        __ceilf(__x)
#   define ceill(__x)        __ceill(__x)
#   define cos(__x)          __cos(__x)
#   define exp(__x)          __exp(__x)
#   define fabs(__x)         __fabs(__x)
#   define floor(__x)        __floor(__x)
#   define floorf(__x)       __floorf(__x)
#   define floorl(__x)       __floorl(__x)
#   define log(__x)          __log(__x)
#   define log10(__x)        __log10(__x)
#   define sin(__x)          __sin(__x)
#   define sqrt(__x)         __sqrt(__x)
#   define tan(__x)          __tan(__x)

#   ifdef _ALL_SOURCE
#     define copysign(x,y)     __copysign(x,y)
#   endif /* _ALL_SOURCE */

# endif /* __cplusplus */

# undef  HUGE_VAL
# define HUGE_VAL (__extension__ 0x1.0p2047)

# if !((defined(__STDC__) && __STDC_VERSION__ < 199901L)) || defined(_ISOC99_SOURCE)
#  ifdef __cplusplus
    static const unsigned int _SINFINITY = 0x7f800000;
    static const unsigned int _SQNAN     = 0x7fc00000;
#   undef  INFINITY
#   define INFINITY  (*((float *)(&_SINFINITY)))
#   undef  NAN
#   define NAN       (*((float *)(&_SQNAN)))      
#  else
#   undef  INFINITY
#   define INFINITY  __builtin_val("0x7f800000", float)  
#   undef  NAN
#   define NAN       __builtin_val("0x7fc00000", float) 
#  endif
#   undef  HUGE_VALF
#   define HUGE_VALF INFINITY
#   undef  HUGE_VALL
#   define HUGE_VALL HUGE_VAL

#   undef  isgreater
#   define isgreater(x, y)                                         \
      (__extension__                                               \
        ({ __typeof__(x) __x = (x); __typeof__(y) __y = (y);       \
           (isnan(__x) || isnan(__y)) ? 0 : (__x) > (__y); }))

#   undef  isgreaterequal
#   define isgreaterequal(x, y)                                    \
      (__extension__                                               \
        ({ __typeof__(x) __x = (x); __typeof__(y) __y = (y);       \
           (isnan(__x) || isnan(__y)) ? 0 : (__x) >= (__y); }))

#   undef  isless
#   define isless(x, y)                                            \
      (__extension__                                               \
        ({ __typeof__(x) __x = (x); __typeof__(y) __y = (y);       \
           (isnan(__x) || isnan(__y)) ? 0 : (__x) < (__y); }))

#   undef  islessequal
#   define islessequal(x, y)                                       \
      (__extension__                                               \
        ({ __typeof__(x) __x = (x); __typeof__(y) __y = (y);       \
           (isnan(__x) || isnan(__y)) ? 0 : (__x) <= (__y); }))

#   undef  islessgreater
#   define islessgreater(x, y)                                     \
      (__extension__                                               \
        ({ __typeof__(x) __x = (x); __typeof__(y) __y = (y);       \
           (isnan(__x) || isnan(__y)) ? 0 : (__x) != (__y); }))

#   undef  isunordered
#   define isunordered(x, y)                                       \
      (__extension__                                               \
        ({ __typeof__(x) __x = (x); __typeof__(y) __y = (y);       \
           (isnan(__x) || isnan(__y)); }))
# endif /* ISOC99 */

#endif /* defined __MATH && ! defined __NO_MATH_INLINES */

#endif /* not __IBM_VACPP_math_h */ 
